﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;

#endregion

namespace LocalizedDataSample {
  partial class LocalizedDataSampleForm : Form {
    public LocalizedDataSampleForm() {
      InitializeComponent();
    }

    private void LocalizedDataSampleForm_Load(object sender, EventArgs e) {

      // Get example values
      double amount = 4.52;
      DateTime date = DateTime.Now;

      // Show localized versions of the example values
      foreach( CultureInfo info in
        CultureInfo.GetCultures(CultureTypes.AllCultures) ) {
        ListViewItem item = listView.Items.Add(info.EnglishName);
        item.SubItems.Add(info.Name);
        if( !info.IsNeutralCulture ) {
          item.SubItems.Add(amount.ToString("C", info.NumberFormat));
          item.SubItems.Add(date.ToString("d", info.DateTimeFormat));
        }
      }
    }

    private void testCulturesButton_Click(object sender, EventArgs e) {
      double amount = 4.52;

      // Show currency using default culture
      MessageBox.Show(amount.ToString("C"),
        Application.CurrentCulture.EnglishName);

      // Change current culture (one way)
      Application.CurrentCulture = new CultureInfo("fr-CA");

      // Change current culture (another way)
      System.Threading.Thread.CurrentThread.CurrentCulture =
        new CultureInfo("fr-CA");

      // Show currency in current culture (Canadian French)
      MessageBox.Show(amount.ToString("C"),
        Application.CurrentCulture.EnglishName);
    }
  }
}