using System;
using System.ComponentModel;
using System.Text;

namespace DataBindingFundamentalsSample {
  public class RaceCarDrivers : BindingList<RaceCarDriver>, IComponent {
    #region IComponent Members

    ISite site = null;

    [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
    [Browsable(false)]
    public ISite Site {
      get { return site; }
      set { site = value; }
    }

    public event EventHandler Disposed;

    public void Dispose() {
      this.Dispose(true);
    }

    protected virtual void Dispose(bool disposing) {
      if( disposing ) {
        lock( this ) {
          if( this.Disposed != null ) this.Disposed(this, EventArgs.Empty);
        }
      }
    }

    #endregion
  }
  
  public class RaceCarDriver : INotifyPropertyChanged {

    private string name;
    private int wins;

    // Needed so DGV can add new rows
    public RaceCarDriver() { }
    
    public RaceCarDriver(string name, int wins) {
      this.name = name;
      this.wins = wins;
    }

    //public event EventHandler NameChanged;
    //public event EventHandler WinsChanged;

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    public string Name {
      get { return this.name; }
      set {
        this.name = value;
        //if( this.NameChanged != null ) this.NameChanged(this, EventArgs.Empty);
        this.OnPropertyChanged("Name");
      }
    }

    public int Wins {
      get { return this.wins; }
      set {
        this.wins = value;
        //if( this.WinsChanged != null ) this.WinsChanged(this, EventArgs.Empty);
        this.OnPropertyChanged("Wins");
      }
    }

    private void OnPropertyChanged(string propertyName) {
      if( this.PropertyChanged != null ) {
        this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
      }
    }
  }
}
