﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using System.IO;
using System.Diagnostics;

namespace TreeViewListViewDragAndDropSample {

  // KeyState Values (not available in WinForms)
  [FlagsAttribute]
  enum KeyState {
    LeftMouse = 1,
    RightMouse = 2,
    ShiftKey = 4,
    CtrlKey = 8,
    MiddleMouse = 16,
    AltKey = 32,
  }

  partial class MainForm : Form {

    private bool dragging = false;

    public MainForm() {
      InitializeComponent();

      #region Create test data

      // Create some test list view items, passing name and image index
      List<ListViewItem> listViewItems = new List<ListViewItem>();
      listViewItems.Add(new ListViewItem("Item 1", 1));
      listViewItems.Add(new ListViewItem("Item 2", 1));
      listViewItems.Add(new ListViewItem("Item 3", 1));
      listViewItems.Add(new ListViewItem("Item 4", 1));
      listViewItems.Add(new ListViewItem("Item 5", 1));

      // Create some test tree view items
      this.treeView.Nodes.AddRange(
        new System.Windows.Forms.TreeNode[] {
          new ItemHolderTreeNode("Node 1", listViewItems),
          new ItemHolderTreeNode("Node 2", null),
          new ItemHolderTreeNode("Node 3", null),
          new ItemHolderTreeNode("Node 4", null),
          new ItemHolderTreeNode("Node 5", null)});

      //// Create some test tree view items
      //ItemHolderTreeNode node1 = new ItemHolderTreeNode("Node 1", listViewItems);
      //node1.Nodes.AddRange(
      //  new System.Windows.Forms.TreeNode[] {
      //    new ItemHolderTreeNode("Node 1.1", null),
      //    new ItemHolderTreeNode("Node 1.2", null),
      //    new ItemHolderTreeNode("Node 1.3", null),
      //    new ItemHolderTreeNode("Node 1.4", null),
      //    new ItemHolderTreeNode("Node 1.5", null)});
      //this.treeView.Nodes.Add(node1);

      //ItemHolderTreeNode node2 = new ItemHolderTreeNode("Node 2", null);
      //node2.Nodes.AddRange(
      //  new System.Windows.Forms.TreeNode[] {
      //    new ItemHolderTreeNode("Node 2.1", null),
      //    new ItemHolderTreeNode("Node 2.2", null),
      //    new ItemHolderTreeNode("Node 2.3", null),
      //    new ItemHolderTreeNode("Node 2.4", null),
      //    new ItemHolderTreeNode("Node 2.5", null)});
      //this.treeView.Nodes.Add(node2);

      //ItemHolderTreeNode node3 = new ItemHolderTreeNode("Node 3", null);
      //node3.Nodes.AddRange(
      //  new System.Windows.Forms.TreeNode[] {
      //    new ItemHolderTreeNode("Node 3.1", null),
      //    new ItemHolderTreeNode("Node 3.2", null),
      //    new ItemHolderTreeNode("Node 3.3", null),
      //    new ItemHolderTreeNode("Node 3.4", null),
      //    new ItemHolderTreeNode("Node 3.5", null)});
      //this.treeView.Nodes.Add(node3);

      //ItemHolderTreeNode node4 = new ItemHolderTreeNode("Node 4", null);
      //node4.Nodes.AddRange(
      //  new System.Windows.Forms.TreeNode[] {
      //    new ItemHolderTreeNode("Node 4.1", null),
      //    new ItemHolderTreeNode("Node 4.2", null),
      //    new ItemHolderTreeNode("Node 4.3", null),
      //    new ItemHolderTreeNode("Node 4.4", null),
      //    new ItemHolderTreeNode("Node 4.5", null)});
      //this.treeView.Nodes.Add(node4);

      //ItemHolderTreeNode node5 = new ItemHolderTreeNode("Node 5", null);
      //node5.Nodes.AddRange(
      //  new System.Windows.Forms.TreeNode[] {
      //    new ItemHolderTreeNode("Node 5.1", null),
      //    new ItemHolderTreeNode("Node 5.2", null),
      //    new ItemHolderTreeNode("Node 5.3", null),
      //    new ItemHolderTreeNode("Node 5.4", null),
      //    new ItemHolderTreeNode("Node 5.5", null)});
      //this.treeView.Nodes.Add(node5);

      #endregion

      // Select first node
      this.treeView.Focus();
    }

    void listView_ItemDrag(object sender, ItemDragEventArgs e) {
      // Don't drag unless left mouse button is down
      if( (e.Button != MouseButtons.Left) ) { return; }

      // Remember source tree node
      TreeNode dragSourceTreeNode = this.treeView.SelectedNode;

      // Get drag data
      ListView.SelectedListViewItemCollection dragData = this.listView.SelectedItems;

      // Set supported drag effects
      DragDropEffects supportedEffects = DragDropEffects.Copy | DragDropEffects.Move;

      // Start a drag and drop operation
      this.dragging = true;
      DragDropEffects dragEffect = this.listView.DoDragDrop(dragData, supportedEffects);
      this.dragging = false;

      // Move if required
      if( dragEffect == DragDropEffects.Move ) {
        foreach( ListViewItem item in this.listView.SelectedItems ) {
          ((ItemHolderTreeNode)dragSourceTreeNode).ListViewItems.Remove(item);
        }
      }

      dragSourceTreeNode = null;

      // Refreshed drop target tree node to display dropped item(s)
      RefreshSelectedTreeNode((ItemHolderTreeNode)this.treeView.SelectedNode);
    }

    void listView_GiveFeedback(object sender, GiveFeedbackEventArgs e) {
      // Select tree node if dragging over one
      Point pt = Cursor.Position;
      TreeNode node = this.GetTreeNodeAtCursor(pt.X, pt.Y);
      if( node != null ) {
        this.treeView.SelectedNode = node;
      }

      // Focus() forces the selection UI to be rendered
      this.treeView.Focus();
    }

    void treeView_DragOver(object sender, DragEventArgs e) {
      // Can't drop unknown data types
      if( !e.Data.GetDataPresent(typeof(ListView.SelectedListViewItemCollection)) ) {
        e.Effect = DragDropEffects.None;
        return;
      }

      // Can't drop outside of tree node
      if( !IsTreeNodeAtCursor(e.X, e.Y) ) {
        e.Effect = DragDropEffects.None;
        return;
      }

      // Show copy or move cursor
      KeyState keyState = (KeyState)e.KeyState;
      if( ((keyState & KeyState.CtrlKey) == KeyState.CtrlKey) ) {
        e.Effect = DragDropEffects.Copy;
      }
      else {
        e.Effect = DragDropEffects.Move;
      }
    }

    void treeView_DragDrop(object sender, DragEventArgs e) {
      // Don't drop if attempting to drop at a non-droppable location
      if( !this.IsTreeNodeAtCursor(e.X, e.Y) ) { return; }

      // Get drag data
      ListView.SelectedListViewItemCollection dragData = (ListView.SelectedListViewItemCollection)e.Data.GetData(typeof(ListView.SelectedListViewItemCollection));
      ItemHolderTreeNode targetNode = (ItemHolderTreeNode)this.GetTreeNodeAtCursor(e.X, e.Y);

      // Move or copy list view item(s) to the drop target tree node
      foreach( ListViewItem item in dragData ) {
        targetNode.ListViewItems.Add(item);
      }
    }

    void treeView_AfterSelect(object sender, TreeViewEventArgs e) {
      // Only refresh list view control if not dragging
      if( this.dragging ) { return; }

      // Refreshed selected tree node to display dropped item(s)
      RefreshSelectedTreeNode((ItemHolderTreeNode)e.Node);
    }

    void RefreshSelectedTreeNode(ItemHolderTreeNode treeNode) {
      if( treeNode == null ) { return; }

      // Remove current list view items
      this.listView.Items.Clear();

      // Add selected tree node's list view items
      this.listView.Items.AddRange(treeNode.ListViewItems.ToArray());
    }

    bool IsTreeNodeAtCursor(int x, int y) {
      return GetTreeNodeAtCursor(x, y) != null;
    }

    TreeNode GetTreeNodeAtCursor(int x, int y) {
      Point pt = this.treeView.PointToClient(new Point(x, y));
      TreeViewHitTestInfo hti = this.treeView.HitTest(pt);
      return hti.Node;
    }
  }
}